@@ -11,7 +11,7 @@ from pymatgen.analysis.phase_diagram import PhaseDiagram
 from pymatgen.util.string import latexify
 from piro.data import GASES, GAS_RELEASE, DEFAULT_GAS_PRESSURES
 from piro.utils import get_v, epitaxy, similarity, update_gases, through_cache
-from piro import MP_API_KEY, RXN_FILES
+from piro import RXN_FILES
 from tqdm.autonotebook import tqdm
 from scipy.special import comb
 
@@ -21,11 +21,28 @@ from scipy.special import comb
 
 
 class SynthesisRoutes:
-    def __init__(self, target_entry_id, confine_to_icsd=True, confine_to_stables=True, hull_distance=np.inf,
-                 simple_precursors=False, explicit_includes=None, allow_gas_release=False,
-                 add_element=None, temperature=298, pressure=1, use_cache=True, exclude_compositions=None,
-                 entries=None, epitaxies=None, similarities=None, sigma=None, transport_constant=None,
-                 custom_target_entry=None, flexible_competition=None):
+    def __init__(
+        self,
+        target_entry_id,
+        confine_to_icsd=True,
+        confine_to_stables=True,
+        hull_distance=np.inf,
+        simple_precursors=False,
+        explicit_includes=None,
+        allow_gas_release=False,
+        add_element=None,
+        temperature=298,
+        pressure=1,
+        use_cache=True,
+        exclude_compositions=None,
+        entries=None,
+        epitaxies=None,
+        similarities=None,
+        sigma=None,
+        transport_constant=None,
+        custom_target_entry=None,
+        flexible_competition=None,
+    ):
         """
         Synthesis reaction route recommendations, derived semi-empirically using the Classical Nucleation Theory
             and high-throughput DFT data.
@@ -36,7 +53,8 @@ class SynthesisRoutes:
             confine_to_stables: Use stable entries only to find. Defaults to True.
             hull_distance (float): Use entries within this distance to hull (eV/atom). Can significantly increase
                 number of possible precursors and slow down the predictions. Ignored if confine_to_stables is True.
-            simple_precursors (bool or int): If True, or integer >0, precursors with fewer components will be considered.
+            simple_precursors (bool or int): If True, or integer >0, precursors with fewer components will
+                be considered.
             explicit_includes (list): list of mp-ids to explicitly include. For example, confine_to_stables may exclude
                 certain common precursors in some systems, if they are not on the convex-hull - this allows such
                 potential precursors to be added to the library.
@@ -89,9 +107,9 @@ class SynthesisRoutes:
         self.plot_data = None
         self.reactions = {}
         if not entries:
-            a = MPRester(MP_API_KEY)
             if not custom_target_entry:
-                _e = a.get_entry_by_material_id(self.target_entry_id)
+                with MPRester() as mpr:
+                    _e = mpr.get_entry_by_material_id(self.target_entry_id)
             else:
                 _e = custom_target_entry
             self.elts = list(_e.composition.as_dict().keys())
@@ -107,12 +125,15 @@ class SynthesisRoutes:
         print("Precursor library ready.")
 
     def get_mp_entries(self):
-        a = MPRester(MP_API_KEY)
-        self.entries = a.get_entries_in_chemsys(self.elts, inc_structure='final',
-                                                property_data=['icsd_ids', 'formation_energy_per_atom'])
+        with MPRester() as mpr:
+            self.entries = mpr.get_entries_in_chemsys(
+                self.elts,
+                inc_structure="final",
+                property_data=["icsd_ids", "formation_energy_per_atom"],
+            )
         for entry in self.entries:
             entry.structure.entry_id = entry.entry_id
-        print('Total # of entries found in this chemistry: ', len(self.entries))
+        print("Total # of entries found in this chemistry: ", len(self.entries))
 
     @property
     def target_entry(self):
@@ -126,16 +147,25 @@ class SynthesisRoutes:
         if self.confine_to_stables:
             precursor_library = list(phased.stable_entries)
         elif self.hull_distance < np.inf:
-            precursor_library = [e for e in self.entries if phased.get_e_above_hull(e) <= self.hull_distance]
+            precursor_library = [
+                e
+                for e in self.entries
+                if phased.get_e_above_hull(e) <= self.hull_distance
+            ]
         else:
             precursor_library = [e for e in self.entries]
         if self.confine_to_icsd:
-            precursor_library = [i for i in precursor_library if i.data['icsd_ids']]
+            precursor_library = [i for i in precursor_library if i.data["icsd_ids"]]
 
         if self.simple_precursors:
-            precursor_library = [i for i in precursor_library
-                                 if len(i.composition.elements) < len(
-                    self.target_entry.composition.elements) - self.simple_precursors + 1]
+            precursor_library = [
+                i
+                for i in precursor_library
+                if len(i.composition.elements)
+                < len(self.target_entry.composition.elements)
+                - self.simple_precursors
+                + 1
+            ]
 
         if self.target_entry in precursor_library:
             precursor_library.pop(precursor_library.index(self.target_entry))
@@ -145,51 +175,80 @@ class SynthesisRoutes:
             for entry_id in self.explicit_includes:
                 try:
                     entry = [e for e in self.entries if e.entry_id == entry_id][0]
-                except:
+                except IndexError:
                     print("Could not find {} in entry list".format(entry_id))
                     continue
                 if entry not in precursor_library:
                     precursor_library.append(entry)
 
         if self.exclude_compositions:
-            precursor_library = [i for i in precursor_library if i.composition.reduced_formula
-                                 not in self.exclude_compositions]
+            precursor_library = [
+                i
+                for i in precursor_library
+                if i.composition.reduced_formula not in self.exclude_compositions
+            ]
 
         self.precursor_library = precursor_library
 
-        print('Total # of precusors materials obeying the provided filters: ',len(precursor_library))
+        print(
+            "Total # of precusors materials obeying the provided filters: ",
+            len(precursor_library),
+        )
         return self.precursor_library
 
     def get_similarities(self):
         if self.use_cache:
-            _similarities = through_cache([s.structure for s in self.precursor_library],
-                                          self.target_entry.structure, type="similarity")
+            _similarities = through_cache(
+                [s.structure for s in self.precursor_library],
+                self.target_entry.structure,
+                type="similarity",
+            )
         else:
-            _similarities = similarity([s.structure for s in self.precursor_library], self.target_entry.structure)
-        self.similarities = dict(zip([i.entry_id for i in self.precursor_library], _similarities))
+            _similarities = similarity(
+                [s.structure for s in self.precursor_library],
+                self.target_entry.structure,
+            )
+        self.similarities = dict(
+            zip([i.entry_id for i in self.precursor_library], _similarities)
+        )
         print("Similarity matrix ready")
         return self.similarities
 
     def get_epitaxies(self):
         if self.use_cache:
-            _epitaxies = through_cache([s.structure for s in self.precursor_library],
-                                       self.target_entry.structure, type="epitaxy")
+            _epitaxies = through_cache(
+                [s.structure for s in self.precursor_library],
+                self.target_entry.structure,
+                type="epitaxy",
+            )
         else:
-            _epitaxies = epitaxy([s.structure for s in self.precursor_library], self.target_entry.structure)
-        self.epitaxies = dict(zip([i.entry_id for i in self.precursor_library], _epitaxies))
+            _epitaxies = epitaxy(
+                [s.structure for s in self.precursor_library],
+                self.target_entry.structure,
+            )
+        self.epitaxies = dict(
+            zip([i.entry_id for i in self.precursor_library], _epitaxies)
+        )
         print("Epitaxies ready")
         return self.epitaxies
 
     def get_reactions(self):
 
-        target_c = get_v(self.target_entry.structure.composition.fractional_composition, self.elts)
+        target_c = get_v(
+            self.target_entry.structure.composition.fractional_composition, self.elts
+        )
 
-        for precursors in tqdm(itertools.combinations(self.precursor_library, len(self.elts)),
-                               total=comb(len(self.precursor_library),len(self.elts))):
+        for precursors in tqdm(
+            itertools.combinations(self.precursor_library, len(self.elts)),
+            total=comb(len(self.precursor_library), len(self.elts)),
+        ):
 
             precursors = list(precursors)
 
-            c = [get_v(e.structure.composition.fractional_composition, self.elts) for e in precursors]
+            c = [
+                get_v(e.structure.composition.fractional_composition, self.elts)
+                for e in precursors
+            ]
             if np.any(np.sum(np.array(c), axis=0) == 0.0):
                 continue
 
@@ -203,7 +262,9 @@ class SynthesisRoutes:
             if np.any(np.abs(coeffs) > 100):
                 continue
 
-            precursor_formulas = np.array([p.structure.composition.reduced_formula for p in precursors])
+            precursor_formulas = np.array(
+                [p.structure.composition.reduced_formula for p in precursors]
+            )
             if len(set(precursor_formulas)) != len(precursor_formulas):
                 continue
 
@@ -219,53 +280,70 @@ class SynthesisRoutes:
                     precursors.pop(i)
                     coeffs = np.delete(coeffs, i)
                     # Update effective rank
-                    c_new = [get_v(e.structure.composition.fractional_composition, self.elts) for e in precursors]
+                    c_new = [
+                        get_v(e.structure.composition.fractional_composition, self.elts)
+                        for e in precursors
+                    ]
                     effective_rank = scipy.linalg.lstsq(np.vstack(c_new).T, target_c)[2]
-            if effective_rank<len(coeffs):
+            if effective_rank < len(coeffs):
                 # Removes under-determined reactions.
                 # print(effective_rank, precursor_formulas, \
                 # [prec_.composition.reduced_formula for prec_ in precursors],coeffs)
                 continue
 
-            label = '_'.join(sorted([e.entry_id for e in precursors]))
+            label = "_".join(sorted([e.entry_id for e in precursors]))
             if label in self.reactions:
                 continue
             else:
-                self.reactions[label] = {'precursors': deepcopy(precursors),
-                                         'coeffs': coeffs,
-                                         'precursor_formulas': np.array([p.structure.composition.reduced_formula
-                                                                         for p in precursors]),
-                                         'precursor_ids': [p.entry_id for p in precursors]
-                                         }
+                self.reactions[label] = {
+                    "precursors": deepcopy(precursors),
+                    "coeffs": coeffs,
+                    "precursor_formulas": np.array(
+                        [p.structure.composition.reduced_formula for p in precursors]
+                    ),
+                    "precursor_ids": [p.entry_id for p in precursors],
+                }
         print("Total # of balanced reactions obtained: ", len(self.reactions))
         return self.reactions
 
     def get_reaction_energy(self, rxn_label, verbose=False):
-        precursors = update_gases(self.reactions[rxn_label]['precursors'], T=self.temperature, P=self.pressure,
-                                  copy=True)
+        precursors = update_gases(
+            self.reactions[rxn_label]["precursors"],
+            T=self.temperature,
+            P=self.pressure,
+            copy=True,
+        )
         # Free energy per atom
-        energies = np.array([e.data['formation_energy_per_atom'] for e in precursors])
-        self.reactions[rxn_label]['energy'] = self.target_entry.data['formation_energy_per_atom'] \
-                                              - np.sum(self.reactions[rxn_label]['coeffs'] * energies)
+        energies = np.array([e.data["formation_energy_per_atom"] for e in precursors])
+        self.reactions[rxn_label]["energy"] = self.target_entry.data[
+            "formation_energy_per_atom"
+        ] - np.sum(self.reactions[rxn_label]["coeffs"] * energies)
         # Enthalpy energy per atom
-        enthalpies = np.array([e.data['enthalpy'] if 'enthalpy' in e.data
-                               else e.data['formation_energy_per_atom'] for e in precursors])
-        self.reactions[rxn_label]['enthalpy'] = self.target_entry.data['formation_energy_per_atom'] \
-                                              - np.sum(self.reactions[rxn_label]['coeffs'] * enthalpies)
+        enthalpies = np.array(
+            [
+                e.data["enthalpy"]
+                if "enthalpy" in e.data
+                else e.data["formation_energy_per_atom"]
+                for e in precursors
+            ]
+        )
+        self.reactions[rxn_label]["enthalpy"] = self.target_entry.data[
+            "formation_energy_per_atom"
+        ] - np.sum(self.reactions[rxn_label]["coeffs"] * enthalpies)
 
-        self.reactions[rxn_label]['temperature'] = self.temperature
+        self.reactions[rxn_label]["temperature"] = self.temperature
 
         if verbose:
-            print('target e: ', self.target_entry.data['formation_energy_per_atom'])
-            print('precursr: ', [e.composition.reduced_formula for e in precursors])
-            print('energies: ', energies)
-            print('coeffs:   ', self.reactions[rxn_label]['coeffs'])
+            print("target e: ", self.target_entry.data["formation_energy_per_atom"])
+            print("precursr: ", [e.composition.reduced_formula for e in precursors])
+            print("energies: ", energies)
+            print("coeffs:   ", self.reactions[rxn_label]["coeffs"])
 
-        return self.reactions[rxn_label]['energy']
+        return self.reactions[rxn_label]["energy"]
 
     @staticmethod
     def f(q):
-        S = 1. - 2 * q
+        S = 1.0 - 2 * q
         if S < -1:
             S = -1
         elif S > 1:
@@ -281,26 +359,39 @@ class SynthesisRoutes:
         return self._transport_constant
 
     def get_nucleation_barrier(self, rxn_label):
-        rx_e = self.reactions[rxn_label]['energy']
+        rx_e = self.reactions[rxn_label]["energy"]
         if rx_e > 0.0:
-            self.reactions[rxn_label]['barrier'] = np.inf
-            self.reactions[rxn_label]['_params'] = None
-            return self.reactions[rxn_label]['barrier']
+            self.reactions[rxn_label]["barrier"] = np.inf
+            self.reactions[rxn_label]["_params"] = None
+            return self.reactions[rxn_label]["barrier"]
 
         target_s = self.target_entry.structure
         delta_Gv = rx_e * target_s.num_sites / target_s.volume
 
-        precursors = self.reactions[rxn_label]['precursors']
-        precursor_formulas = self.reactions[rxn_label]['precursor_formulas']
+        precursors = self.reactions[rxn_label]["precursors"]
+        # precursor_formulas = self.reactions[rxn_label]["precursor_formulas"]
 
-        q_epi = min([self.epitaxies[i.entry_id] for i in precursors
-                     if not i.structure.composition.reduced_formula in GASES]) / 1000.0
+        q_epi = (
+            min(
+                [
+                    self.epitaxies[i.entry_id]
+                    for i in precursors
+                    if i.structure.composition.reduced_formula not in GASES
+                ]
+            )
+            / 1000.0
+        )
         q_epi = min(q_epi, 1.0)
 
         try:
-            q_sim = min([self.similarities[i.entry_id] for i in precursors
-                         if int(
-                    self.similarities[i.entry_id]) != -1 and not i.structure.composition.reduced_formula in GASES])
+            q_sim = min(
+                [
+                    self.similarities[i.entry_id]
+                    for i in precursors
+                    if int(self.similarities[i.entry_id]) != -1
+                    and i.structure.composition.reduced_formula not in GASES
+                ]
+            )
         except:
             # better handling needed
             q_sim = 1.0
@@ -309,51 +400,74 @@ class SynthesisRoutes:
         G_star = 16 / 3 * np.pi * self.sigma ** 3 * f_t / delta_Gv ** 2
         E_d = self.transport_constant * q_sim
 
-        self.reactions[rxn_label]['barrier'] = G_star + E_d
-        self.reactions[rxn_label]['_params'] = {'f_t': f_t, 'G_star': G_star, 'E_d': E_d, 'q_sim': q_sim,
-                                                'q_epi': q_epi}
+        self.reactions[rxn_label]["barrier"] = G_star + E_d
+        self.reactions[rxn_label]["_params"] = {
+            "f_t": f_t,
+            "G_star": G_star,
+            "E_d": E_d,
+            "q_sim": q_sim,
+            "q_epi": q_epi,
+        }
 
-        return self.reactions[rxn_label]['barrier']
+        return self.reactions[rxn_label]["barrier"]
 
     def get_rxn_summary(self, rxn_label):
-        coeffs = self.reactions[rxn_label]['coeffs']
-        num_sites = sum(self.target_entry.structure.composition.reduced_composition.as_dict().values())
+        coeffs = self.reactions[rxn_label]["coeffs"]
+        num_sites = sum(
+            self.target_entry.structure.composition.reduced_composition.as_dict().values()
+        )
         _coeffs = []
         for i in range(len(coeffs)):
-            p = self.reactions[rxn_label]['precursors'][i]
-            _coeffs.append(coeffs[i] / sum(p.structure.composition.reduced_composition.as_dict().values()) * num_sites)
+            p = self.reactions[rxn_label]["precursors"][i]
+            _coeffs.append(
+                coeffs[i]
+                / sum(p.structure.composition.reduced_composition.as_dict().values())
+                * num_sites
+            )
         _coeffs = np.round(_coeffs, decimals=4)
-        report = ' + '.join(
+        report = " + ".join(
             [
-                i + ' ' + j for i, j in list(
-                zip([str(c) for c in _coeffs], [p.structure.composition.reduced_formula + '(' + p.entry_id + ')'
-                                                for p in self.reactions[rxn_label]['precursors']]))
+                i + " " + j
+                for i, j in list(
+                    zip(
+                        [str(c) for c in _coeffs],
+                        [
+                            p.structure.composition.reduced_formula
+                            + "("
+                            + p.entry_id
+                            + ")"
+                            for p in self.reactions[rxn_label]["precursors"]
+                        ],
+                    )
+                )
             ]
         )
-        self.reactions[rxn_label]['summary'] = report
+        self.reactions[rxn_label]["summary"] = report
         return report
 
     def get_competing_phases(self, rxn_label, confine_to_icsd=True):
 
-        precursors = self.reactions[rxn_label]['precursors']
+        precursors = self.reactions[rxn_label]["precursors"]
         precursor_ids = [i.entry_id for i in precursors]
         _competing = []
         _competing_rxe = []
 
         for entry in self.entries:
             if confine_to_icsd:
-                if not entry.data['icsd_ids']:
+                if not entry.data["icsd_ids"]:
                     continue
             if self.flexible_competition:
                 s1 = set(entry.composition.as_dict().keys())
                 s2 = set(self.target_entry.structure.composition.as_dict().keys())
-                if not (s1.issubset(s2)
-                        and (len(s2)-self.flexible_competition <= len(s1) <= len(s2))
-                        ):
+                if not (
+                    s1.issubset(s2)
+                    and (len(s2) - self.flexible_competition <= len(s1) <= len(s2))
+                ):
                     continue
             else:
                 if not set(self.target_entry.composition.as_dict().keys()).issubset(
-                        set(entry.structure.composition.as_dict().keys())):
+                    set(entry.structure.composition.as_dict().keys())
+                ):
                     continue
             if entry.entry_id in precursor_ids:
                 continue
@@ -362,7 +476,9 @@ class SynthesisRoutes:
             competing_target_entry = entry
 
             elts_precs = set()
-            for s in [set(p.structure.composition.as_dict().keys()) for p in precursors]:
+            for s in [
+                set(p.structure.composition.as_dict().keys()) for p in precursors
+            ]:
                 elts_precs = elts_precs.union(s)
             if not set(entry.composition.as_dict().keys()).issubset(elts_precs):
                 # print(entry.composition)
@@ -370,10 +486,18 @@ class SynthesisRoutes:
 
             elts_precs = sorted(list(elts_precs))
 
-            target_c = get_v(competing_target_entry.structure.composition.fractional_composition, elts_precs)
-            c = [get_v(e.structure.composition.fractional_composition, elts_precs) for e in precursors]
+            target_c = get_v(
+                competing_target_entry.structure.composition.fractional_composition,
+                elts_precs,
+            )
+            c = [
+                get_v(e.structure.composition.fractional_composition, elts_precs)
+                for e in precursors
+            ]
 
-            precursor_formulas = np.array([p.structure.composition.reduced_formula for p in precursors])
+            precursor_formulas = np.array(
+                [p.structure.composition.reduced_formula for p in precursors]
+            )
 
             # trying to solve for compound fractions.
             try:
@@ -385,7 +509,11 @@ class SynthesisRoutes:
                     if x[1] != 1:
                         continue
                 except:
-                    print('     failed:', competing_target_entry.composition.reduced_formula, precursor_formulas)
+                    print(
+                        "     failed:",
+                        competing_target_entry.composition.reduced_formula,
+                        precursor_formulas,
+                    )
                     print(np.vstack(c).T, target_c)
                     continue
             if np.any(coeffs < 0.0):
@@ -394,7 +522,9 @@ class SynthesisRoutes:
                 else:
                     if not set(precursor_formulas[coeffs < 0.0]).issubset(GAS_RELEASE):
                         continue
-            _precursors = update_gases(precursors, T=self.temperature, P=self.pressure, copy=True)
+            _precursors = update_gases(
+                precursors, T=self.temperature, P=self.pressure, copy=True
+            )
 
             for i in sorted(range(len(coeffs)), reverse=True):
                 if np.abs(coeffs[i]) < 0.00001:
@@ -409,28 +539,47 @@ class SynthesisRoutes:
                     continue
             except:
                 continue
-            energies = np.array([e.data['formation_energy_per_atom'] for e in _precursors])
-            rx_e = competing_target_entry.data['formation_energy_per_atom'] - np.sum(coeffs * energies)
+            energies = np.array(
+                [e.data["formation_energy_per_atom"] for e in _precursors]
+            )
+            rx_e = competing_target_entry.data["formation_energy_per_atom"] - np.sum(
+                coeffs * energies
+            )
             if rx_e < 0.0:
                 _competing.append(competing_target_entry.entry_id)
                 _competing_rxe.append((rx_e))
-        self.reactions[rxn_label]['competing'] = _competing
-        self.reactions[rxn_label]['competing_rxe'] = _competing_rxe
-        self.reactions[rxn_label]['n_competing'] = len(_competing)
+        self.reactions[rxn_label]["competing"] = _competing
+        self.reactions[rxn_label]["competing_rxe"] = _competing_rxe
+        self.reactions[rxn_label]["n_competing"] = len(_competing)
         return len(_competing)
 
-    def recommend_routes(self, temperature=298, pressure=None, allow_gas_release=False,
-                         max_component_precursors=0, show_fraction_known_precursors=True,
-                         show_known_precursors_only=False, confine_competing_to_icsd=True,
-                         display_peroxides=True, display_superoxides=True, w=None, h=None,
-                         xrange=None, yrange=None, add_pareto=False, custom_text=''):
+    def recommend_routes(
+        self,
+        temperature=298,
+        pressure=None,
+        allow_gas_release=False,
+        max_component_precursors=0,
+        show_fraction_known_precursors=True,
+        show_known_precursors_only=False,
+        confine_competing_to_icsd=True,
+        display_peroxides=True,
+        display_superoxides=True,
+        w=None,
+        h=None,
+        xrange=None,
+        yrange=None,
+        add_pareto=False,
+        custom_text="",
+    ):
         if not pressure:
             pressure = self.pressure
         if not (
-                self.temperature == temperature and self.pressure == pressure and
-                self.allow_gas_release == allow_gas_release and
-                self.confine_competing_to_icsd == confine_competing_to_icsd and
-                self.reactions):
+            self.temperature == temperature
+            and self.pressure == pressure
+            and self.allow_gas_release == allow_gas_release
+            and self.confine_competing_to_icsd == confine_competing_to_icsd
+            and self.reactions
+        ):
             self.temperature = temperature
             self.pressure = pressure if pressure else self.pressure
             self.allow_gas_release = allow_gas_release
@@ -440,56 +589,112 @@ class SynthesisRoutes:
                 self.get_reaction_energy(rxn_label)
                 self.get_nucleation_barrier(rxn_label)
                 self.get_rxn_summary(rxn_label)
-                self.get_competing_phases(rxn_label, confine_to_icsd=confine_competing_to_icsd)
+                self.get_competing_phases(
+                    rxn_label, confine_to_icsd=confine_competing_to_icsd
+                )
             self.check_if_known_precursors()
 
-        self.plot_data = pd.DataFrame.from_dict(self.reactions, orient='index')[['n_competing', "barrier", "summary",
-                                                                                 "energy", "enthalpy",
-                                                                                 "exp_precursors", "precursor_formulas"]]
+        self.plot_data = pd.DataFrame.from_dict(self.reactions, orient="index")[
+            [
+                "n_competing",
+                "barrier",
+                "summary",
+                "energy",
+                "enthalpy",
+                "exp_precursors",
+                "precursor_formulas",
+            ]
+        ]
         if max_component_precursors:
-            allowed_precursor_ids = [i.entry_id for i in self.precursor_library
-                                     if len(set(i.composition.as_dict().keys()).difference(self.add_element))
-                                     <= max_component_precursors or i.composition.reduced_formula in GAS_RELEASE]
+            allowed_precursor_ids = [
+                i.entry_id
+                for i in self.precursor_library
+                if len(set(i.composition.as_dict().keys()).difference(self.add_element))
+                <= max_component_precursors
+                or i.composition.reduced_formula in GAS_RELEASE
+            ]
             display_reactions = []
             for r in self.plot_data.index.to_list():
-                if set(r.split('_')).issubset(set(allowed_precursor_ids)):
+                if set(r.split("_")).issubset(set(allowed_precursor_ids)):
                     display_reactions.append(r)
             self.plot_data = self.plot_data.loc[display_reactions]
 
         if not display_peroxides:
-            peroxides = {'Li2O2', 'K2O2', 'BaO2', 'Rb2O2', 'Cs2O2', 'Na2O2', 'SrO2',
-                         'CaO2','MgO2', 'ZnO2', 'CdO2', 'HgO2'}
+            peroxides = {
+                "Li2O2",
+                "K2O2",
+                "BaO2",
+                "Rb2O2",
+                "Cs2O2",
+                "Na2O2",
+                "SrO2",
+                "CaO2",
+                "MgO2",
+                "ZnO2",
+                "CdO2",
+                "HgO2",
+            }
             allowed_rows = []
             for i in range(len(self.plot_data)):
-                if not peroxides.intersection(set(self.plot_data["precursor_formulas"][i].tolist())):
+                if not peroxides.intersection(
+                    set(self.plot_data["precursor_formulas"][i].tolist())
+                ):
                     allowed_rows.append((i))
             self.plot_data = self.plot_data.iloc[allowed_rows]
 
         if not display_superoxides:
-            superoxides = {'LiO2', 'NaO2', 'KO2', 'RbO2', 'CsO2'}
+            superoxides = {"LiO2", "NaO2", "KO2", "RbO2", "CsO2"}
             allowed_rows = []
             for i in range(len(self.plot_data)):
-                if not superoxides.intersection(set(self.plot_data["precursor_formulas"][i].tolist())):
+                if not superoxides.intersection(
+                    set(self.plot_data["precursor_formulas"][i].tolist())
+                ):
                     allowed_rows.append((i))
             self.plot_data = self.plot_data.iloc[allowed_rows]
 
         color = "exp_precursors" if show_fraction_known_precursors else None
 
         if show_known_precursors_only:
-            self.plot_data = self.plot_data[ self.plot_data["exp_precursors"].astype(float) == 1.0 ]
-        fig = px.scatter(self.plot_data, x="n_competing", y="barrier", hover_data=["summary"],
-                             color=color, width=w, height=h, template='simple_white')
+            self.plot_data = self.plot_data[
+                self.plot_data["exp_precursors"].astype(float) == 1.0
+            ]
+        fig = px.scatter(
+            self.plot_data,
+            x="n_competing",
+            y="barrier",
+            hover_data=["summary"],
+            color=color,
+            width=w,
+            height=h,
+            template="simple_white",
+        )
         for i in fig.data:
             i.marker.size = 10
         fig.update_layout(
-            yaxis={'title': 'Nucleation barrier (a.u.)', 'ticks': 'inside', 'mirror': True, 'showline': True},
-            xaxis={'title': 'Number of competing phases', 'ticks': 'inside', 'mirror': True, 'showline': True},
-            font={'size': 13},
-            title=r"Target: " + self.target_entry.structure.composition.reduced_formula + custom_text,
+            yaxis={
+                "title": "Nucleation barrier (a.u.)",
+                "ticks": "inside",
+                "mirror": True,
+                "showline": True,
+            },
+            xaxis={
+                "title": "Number of competing phases",
+                "ticks": "inside",
+                "mirror": True,
+                "showline": True,
+            },
+            font={"size": 13},
+            title=r"Target: "
+            + self.target_entry.structure.composition.reduced_formula
+            + custom_text,
             title_font_size=15,
-            title_x=0.5)
+            title_x=0.5,
+        )
         fig.update_traces(
-            marker=dict(size=12, line=dict(width=2, color='DarkSlateGrey'), opacity=0.8),  selector=dict(mode='markers')
+            marker=dict(
+                size=12, line=dict(width=2, color="DarkSlateGrey"), opacity=0.8
+            ),
+            selector=dict(mode="markers"),
         )
         if xrange:
             fig.update_xaxes(range=xrange)
@@ -498,28 +703,36 @@ class SynthesisRoutes:
 
         if add_pareto:
             import plotly.graph_objects as go
+
             _pareto_data = self.topsis().loc[self.get_pareto_front()]
-            _x = _pareto_data['n_competing']
-            _y = _pareto_data['barrier']
-            fig.add_trace(go.Scatter(
-                x=_x,
-                y=_y,
-                line=dict(color='firebrick', width=2)
-                # connectgaps=True
-            ))
-            fig.add_trace(go.Scatter(
-                x=[_x[0], _x[0], None, _x[-1], self.topsis()['n_competing'].max()],
-                y=[_y[0], self.topsis()['barrier'].max(), None, _y[-1],_y[-1]],
-                line=dict(color='firebrick', width=2, dash='dash'),
-                connectgaps=False
-            ))
+            _x = _pareto_data["n_competing"]
+            _y = _pareto_data["barrier"]
+            fig.add_trace(
+                go.Scatter(
+                    x=_x,
+                    y=_y,
+                    line=dict(color="firebrick", width=2)
+                    # connectgaps=True
+                )
+            )
+            fig.add_trace(
+                go.Scatter(
+                    x=[_x[0], _x[0], None, _x[-1], self.topsis()["n_competing"].max()],
+                    y=[_y[0], self.topsis()["barrier"].max(), None, _y[-1], _y[-1]],
+                    line=dict(color="firebrick", width=2, dash="dash"),
+                    connectgaps=False,
+                )
+            )
 
             fig.update_layout(showlegend=False)
         return fig
 
     def get_precursor_formulas(self, include_ids=True):
         if include_ids:
-            return [(p.entry_id, p.composition.reduced_formula) for p in self.precursor_library]
+            return [
+                (p.entry_id, p.composition.reduced_formula)
+                for p in self.precursor_library
+            ]
         else:
             return [p.composition.reduced_formula for p in self.precursor_library]
 
@@ -533,40 +746,65 @@ class SynthesisRoutes:
         """
         if isinstance(formulas, str):
             formulas = list(formulas)
-        return sorted([(self.reactions[i]['barrier'], self.reactions[i]['summary'], self.reactions[i]['n_competing'],i)
-                for i in self.reactions if all([formula in self.reactions[i]['summary'] for formula in formulas])
-                       ])
+        return sorted(
+            [
+                (
+                    self.reactions[i]["barrier"],
+                    self.reactions[i]["summary"],
+                    self.reactions[i]["n_competing"],
+                    i,
+                )
+                for i in self.reactions
+                if all(
+                    [formula in self.reactions[i]["summary"] for formula in formulas]
+                )
+            ]
+        )
 
     def check_if_known_precursors(self):
-        with open(os.path.join(RXN_FILES,"experimental_precursors_KononovaSciData.json"), 'r') as f:
+        with open(
+            os.path.join(RXN_FILES, "experimental_precursors_KononovaSciData.json"), "r"
+        ) as f:
             exp_precursors = set(json.load(f))
         exp_precursors = exp_precursors.union(set(self.explicit_includes))
 
         for i in self.reactions:
-            ids = [self.reactions[i]['precursor_ids'][j] for j in range(len(self.reactions[i]['precursor_ids']))
-                   if self.reactions[i]['precursor_formulas'][j] not in GASES]
+            ids = [
+                self.reactions[i]["precursor_ids"][j]
+                for j in range(len(self.reactions[i]["precursor_ids"]))
+                if self.reactions[i]["precursor_formulas"][j] not in GASES
+            ]
             frac = len(set(ids).intersection(exp_precursors)) / len(ids)
-            self.reactions[i]['exp_precursors'] = str(np.round(frac,decimals=4))
+            self.reactions[i]["exp_precursors"] = str(np.round(frac, decimals=4))
 
     def get_pareto_front(self):
         """
         Returns: list of reaction labels on the pareto front
         """
-        x = self.plot_data[self.plot_data['barrier'] < np.inf].sort_values(by=['n_competing',
-                                                                               'barrier'])[['n_competing', 'barrier']]
-        y = x.groupby(by=['n_competing'], as_index=False).min()
+        x = self.plot_data[self.plot_data["barrier"] < np.inf].sort_values(
+            by=["n_competing", "barrier"]
+        )[["n_competing", "barrier"]]
+        y = x.groupby(by=["n_competing"], as_index=False).min()
         rows = list(y.iterrows())
         front = []
         barrier_front = []
         for row in rows:
-            n_competing = row[1]['n_competing']
-            barrier = row[1]['barrier']
+            n_competing = row[1]["n_competing"]
+            barrier = row[1]["barrier"]
             if rows.index(row) == 0:
-                front.append(x.index[(x['barrier'] == barrier) & (x['n_competing'] == n_competing)][0])
+                front.append(
+                    x.index[
+                        (x["barrier"] == barrier) & (x["n_competing"] == n_competing)
+                    ][0]
+                )
                 barrier_front.append(barrier)
                 continue
             if barrier < barrier_front[-1]:
-                front.append(x.index[(x['barrier'] == barrier) & (x['n_competing'] == n_competing)][0])
+                front.append(
+                    x.index[
+                        (x["barrier"] == barrier) & (x["n_competing"] == n_competing)
+                    ][0]
+                )
                 barrier_front.append(barrier)
         return front
 
@@ -575,19 +813,19 @@ class SynthesisRoutes:
         Returns a ranked list of reactions based on TOPSIS method for multiobjective optimization.
         Returns:
         """
-        x = self.plot_data[['n_competing','barrier']]
-        x = x[x['barrier'] < np.inf]
+        x = self.plot_data[["n_competing", "barrier"]]
+        x = x[x["barrier"] < np.inf]
         xsum = np.sqrt((x ** 2).sum())
         mu = x / xsum
         positive_ideal = mu.min()
         negative_ideal = mu.max()
         d_pos_ideal = np.sqrt(((mu - positive_ideal) ** 2).sum(axis=1))
         d_neg_ideal = np.sqrt(((mu - negative_ideal) ** 2).sum(axis=1))
-        x['topsis_score'] = d_neg_ideal / (d_pos_ideal + d_neg_ideal)
-        x = x.sort_values(by='topsis_score', ascending=False)
+        x["topsis_score"] = d_neg_ideal / (d_pos_ideal + d_neg_ideal)
+        x = x.sort_values(by="topsis_score", ascending=False)
         result = self.plot_data.loc[x.index]
-        result['topsis_score'] = x['topsis_score']
+        result["topsis_score"] = x["topsis_score"]
 
         if latex:
-            result['summary'] = result['summary'].apply(latexify)
+            result["summary"] = result["summary"].apply(latexify)
         return result
